import pandas as pd
import numpy as np
import datetime
import time
from matplotlib import pyplot as plt
import pymannkendall as mk
from LIB.MIDDLE.CellStateEstimation.Common.V1_0_1 import BatParam

class SafetyWarning:
    def __init__(self,sn,celltype,df_short,df_uniform,df_soh,df_fault_ram_sn):  #参数初始化

        self.sn=sn
        self.celltype=celltype
        self.param=BatParam.BatParam(celltype)
        self.df_short=df_short
        self.df_uniform=df_uniform
        # self.OutLineVol_Rate=OutLineVol_Rate
        self.df_soh=df_soh
        self.df_warning_ram=df_fault_ram_sn.copy()
    
    def diag(self):
        df_res_new, df_res_chg=self._warning_diag()
        return df_res_new, df_res_chg

    #电池热安全预警诊断功能.................................................................................................
    def _warning_diag(self):

        df_res_new=pd.DataFrame(columns=['start_time', 'end_time', 'product_id', 'code', 'level', 'info', 'advice'])
        df_res_chg=pd.DataFrame(columns=['start_time', 'end_time', 'product_id', 'code', 'level', 'info', 'advice'])

        time_now=datetime.datetime.now()
        time_now=time_now.strftime('%Y-%m-%d %H:%M:%S')
        end_time='0000-00-00 00:00:00'

         #参数初始化.......................................
        cellshortfault=0
        cellshortfault1=0
        uniformfault=0

        #计算漏电流离群度 &  拆分电芯漏电流值---------------------------------------
        df_short=pd.DataFrame()
        df_short1=pd.DataFrame()
        if not self.df_short.empty:
            df_short1=self.df_short[self.df_short['method']==4]
            df_short1.reset_index(drop=True,inplace=True)
            self.df_short=self.df_short.drop(self.df_short[self.df_short['method']==4].index)
            self.df_short.reset_index(drop=True,inplace=True)
            short_name=['cellshort'+str(i) for i in range(1,self.param.CellVoltNums+1)]
            df_short_current=pd.DataFrame(columns=short_name)
            self.df_short['cellshort_sigma']=0
            for i in range(len(self.df_short)):
                cellshort=eval(self.df_short.loc[i,'short_current'])
                df_short_current.loc[i]=cellshort
                cellshort_std=np.std(cellshort)
                cellshort_mean=np.mean(cellshort)
                self.df_short.loc[i,'cellshort_sigma']=str(list((cellshort-cellshort_mean)/cellshort_std))
            df_short=pd.concat([self.df_short,df_short_current],axis=1)
        
        #拆分电压排名----------------------------------------------------------------------------------------
        self.df_uniform=self.df_uniform.drop(self.df_uniform[self.df_uniform['cellvolt_rank']=='0'].index)
        self.df_uniform.reset_index(inplace=True,drop=True)
        if not self.df_uniform.empty:
            rank_name=['cellvolt_rank'+str(i) for i in range(1,self.param.CellVoltNums+1)]
            df_rank=pd.DataFrame(columns=rank_name)
            for i in range(len(self.df_uniform)):
                df_rank.loc[i]=eval(self.df_uniform.loc[i,'cellvolt_rank'])
           
    
        if len(df_short)>1.5:     #超过2行
            #出现两次漏电流<阈值...........................................................................
            for i in range(self.param.CellVoltNums):
                cellshort=df_short['cellshort'+str(i+1)]
                index_list=cellshort[cellshort<self.param.LeakCurrentLv2].index
                if len(index_list)>1:   
                    for j in range(1,len(index_list)):
                        if (df_short.loc[index_list[j],'time_st']-df_short.loc[index_list[j-1],'time_st']).total_seconds()<10*24*3600:
                            cellshort_sigma1=eval(df_short.loc[index_list[j],'cellshort_sigma'])
                            cellshort_sigma2=eval(df_short.loc[index_list[j-1],'cellshort_sigma'])
                            if cellshort_sigma1[i]<-3 or cellshort_sigma2[i]<-3:
                                cellshortfault=1
                                cellshort_num=i+1
                            else:
                                cellshortfault=0                  
                        else:
                            cellshortfault=0
                else:
                    cellshortfault=0
        
            #漏电流的和满足且最大漏电流值<阈值.........................................................................
            cellshort_sum=list(df_short_current.sum())
            if min(cellshort_sum)<2*self.param.LeakCurrentLv2:
                cellshort_num=cellshort_sum.index(min(cellshort_sum))+1
                cellshort=df_short_current['cellshort'+str(cellshort_num)]   #获取内短路电芯的所有漏电流值，且最小值<阈值
                cellshort_last = cellshort.iloc[-1] #最近一次的漏电流
                cellshort_sum.remove(min(cellshort_sum))    #删除最小值，并判断其他所有电芯内短路电流>阈值
                if min(cellshort)<self.param.LeakCurrentLv2 and min(cellshort_sum)>self.param.LeakCurrentLv2 and cellshort_last<self.param.LeakCurrentLv2/2:
                    cellshortfault=1
                else:
                    cellshortfault=0
            else:
                cellshortfault=0
            
        elif len(df_short)>0.5: #漏电流只有一行
            cellshort=eval(self.df_short.loc[0,'short_current'])
            cellshort1=cellshort
            cellshort1.remove(min(cellshort))
            if min(cellshort)<self.param.LeakCurrentLv2*3 and min(cellshort1)>self.param.LeakCurrentLv2:
                cellshortfault=1
                cellshort_num=list(cellshort).index(min(cellshort))+1
            else:
                cellshortfault=0
        
        else:
            cellshortfault=0  
            
        #充电电压判断--------------------------------------------
        if not df_short1.empty:
            for i in range(3):
                deltvolt_rst=np.array([0]*self.param.CellVoltNums)
                for j in range(len(df_short1)):
                    try:
                        deltvolt=eval(df_short1.loc[j,'short_current'])
                        deltvolt_rst=deltvolt_rst+np.array(deltvolt[i]) #所有结果累加
                    except:
                        continue
                
                #获取压降最大电芯的最近一次压降值
                try:
                    deltvolt_last = eval(df_short1.iloc[-1]['short_current'])
                    deltvolt_last_list = list(deltvolt_last[i])
                except:
                    deltvolt_last_list = [0]*self.param.CellVoltNums
                
                # 故障判断:1.压降累计最大>6mV，次大<3mV，2.最近一次对应电芯的压降>3mV，且离群
                if min(deltvolt_rst)<-6:
                    deltvolt_rst1=list(deltvolt_rst)
                    deltvolt_min_index = deltvolt_rst1.index(min(deltvolt_rst))
                    deltvolt_last_list1 = deltvolt_last_list.copy()
                    deltvolt_last_list1.pop(deltvolt_min_index)
                    deltvolt_rst1.remove(min(deltvolt_rst))
                    if min(deltvolt_rst1)>-3 and deltvolt_last_list[deltvolt_min_index]<-3 and min(deltvolt_last_list1)>-2.5:
                        cellshortfault1=1
                        cellshort_num=deltvolt_min_index+1
                        break
                # elif min(deltvolt_rst)<-6:
                #     deltvolt_rst1=list(deltvolt_rst)
                #     deltvolt_rst1.remove(min(deltvolt_rst))
                #     if min(deltvolt_rst1)>-3:
                #         cellshortfault1=1
                #         cellshort_num=list(deltvolt_rst).index(min(deltvolt_rst))+1
                else:
                    cellshortfault1=0
            
        #漏电流热失控预警确认.......................................................................................
        if not 'C490' in list(self.df_warning_ram['code']):  #当前故障中没有该故障，则判断是否发生该故障
            if cellshortfault==1:
                faultcode='C490'
                faultlv=4
                faultinfo='电芯{}发生内短路'.format([cellshort_num])
                faultadvice='请立刻召回电池包，更换模组'
                df_res_new.loc[0]=[time_now, end_time, self.sn, faultcode, faultlv, faultinfo, faultadvice]
            else:
                pass
        else:
            if cellshortfault==1 or cellshortfault1==1:
                pass
            else:
                df_res_chg=self.df_warning_ram
                df_res_chg.loc[df_res_chg[df_res_chg['code']=='C490'].index, 'end_time'] = time_now
        
        if not 'C490' in list(self.df_warning_ram['code']):  #当前故障中没有该故障，则判断是否发生该故障
            if cellshortfault1==1:
                faultcode='C490'
                faultlv=4
                faultinfo='电芯{}发生内短路00'.format([cellshort_num])
                faultadvice='请立刻召回电池包，更换模组'
                df_res_new.loc[0]=[time_now, end_time, self.sn, faultcode, faultlv, faultinfo, faultadvice]
            else:
                pass
        else:
            if cellshortfault==1 or cellshortfault1==1:
                pass
            else:
                df_res_chg=self.df_warning_ram
                df_res_chg.loc[df_res_chg[df_res_chg['code']=='C490'].index, 'end_time'] = time_now
        
        return df_res_new, df_res_chg
